package com.nisum.products_producer.controller;

import com.nisum.products_producer.dto.ProductDto;

import com.nisum.products_producer.exception.InvalidPriceException;
import com.nisum.products_producer.exception.InvalidProductNameException;
import com.nisum.products_producer.service.ProductService;
import jakarta.validation.Valid;
import jakarta.validation.ValidationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping("/product")
public class ProductController {


    @Autowired
    private ProductService productService;


    @PostMapping("/saveProduct")
    public Mono<ResponseEntity<ProductDto>> addProduct(@RequestBody  Mono<ProductDto> productDtoMono) {
        return productDtoMono
                .flatMap(productDto -> {
                    // Check if the name is empty or null
                    if (productDto.getName() == null || productDto.getName().trim().isEmpty()) {
                        return Mono.error(new InvalidProductNameException("Product name cannot be empty."));
                    }

                    // Allow letters and numbers but no special characters
                    if (productDto.getName().matches("\\d+")) {
                        return Mono.error(new InvalidProductNameException("Product name cannot be only numbers."));
                    }

                    // Check if the price is valid
                    if (productDto.getPrice() == null || productDto.getPrice() < 0.01) {
                        return Mono.error(new InvalidPriceException("Price must be a positive decimal number."));
                    }

                    return productService.saveProduct(Mono.just(productDto))
                            .map(ResponseEntity::ok)
                            .defaultIfEmpty(ResponseEntity.badRequest().build());
                });
    }
    //

    @GetMapping("/getAllProduct")
    public ResponseEntity<Flux<ProductDto>> getAllProducts() {
        return ResponseEntity.ok(productService.getAllProducts());
    }
    // pagination we can implement for limiting of fetching the records
}
