package com.nisum.products_producer.service;

import com.nisum.products_producer.dto.ProductDto;

import com.nisum.products_producer.exception.DuplicateProductException;
import com.nisum.products_producer.repository.ProductRepository;
import com.nisum.products_producer.util.AppUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class ProductServiceImpl implements ProductService{

//    @Autowired
    private final ProductRepository productRepository;

    public ProductServiceImpl(ProductRepository productRepository) { // Constructor Injection
        this.productRepository = productRepository;
    }


    @Override
    public Mono<ProductDto> saveProduct(Mono<ProductDto> productDto) {


        return productDto
                .map(AppUtil::toEntity)
                .flatMap(productRepository::save)
                .map(AppUtil::toDto)
                .onErrorResume(DuplicateKeyException.class, ex -> {
                    return Mono.error(new DuplicateProductException("Product already exists with the same name and price!"));

                });
    }

    @Override
    public Flux<ProductDto> getAllProducts() {
        return productRepository.findAll()

                .map(AppUtil::toDto);  // Convert Product -> ProductDto
    }
}
