package com.nisum.products_producer.controller;

import com.nisum.products_producer.dto.ProductDto;
import com.nisum.products_producer.service.ProductService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.WebFluxTest;

import org.springframework.http.MediaType;
import org.springframework.test.web.reactive.server.WebTestClient;
import reactor.core.publisher.Mono;

import static org.mockito.Mockito.*;
import static org.assertj.core.api.Assertions.assertThat;

@WebFluxTest(ProductController.class)
@ExtendWith(MockitoExtension.class)
class ProductControllerTest {

    @Mock
    private ProductService productService;

    @InjectMocks
    private ProductController productController;
    private WebTestClient webTestClient;

    @BeforeEach
    void setUp() {
        webTestClient = WebTestClient.bindToController(productController).build();
    }

    @Test
    void addProductTest() {
        // Given: Mock input product
        ProductDto productDto = new ProductDto();
        productDto.setId("101");
        productDto.setName("iPhone 14");
        productDto.setPrice(999.99);

        // Mock service behavior
        when(productService.saveProduct(Mockito.any(Mono.class))).thenReturn(Mono.just(productDto));

        // When: Calling the endpoint
        webTestClient.post()
                .uri("/product/saveProduct")
                .contentType(MediaType.APPLICATION_JSON)
                .bodyValue(productDto)
                .exchange()
                .expectStatus().isOk()
                .expectBody(ProductDto.class)
                .value(response -> {
                    assertThat(response.getId()).isEqualTo("101");
                    assertThat(response.getName()).isEqualTo("iPhone 14");
                    assertThat(response.getPrice()).isEqualTo(999.99);
                });

        // Verify interaction with service
        verify(productService, times(1)).saveProduct(any());
    }
}
